/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.multishop.mapper;

import com.tmerclub.cloud.multishop.dto.IndexImgDTO;
import com.tmerclub.cloud.multishop.model.IndexImg;
import com.tmerclub.cloud.multishop.vo.IndexImgVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 轮播图
 *
 * @author YXF
 * @date 2020-11-24 16:38:32
 */
public interface IndexImgMapper {

    /**
     * 获取轮播图列表
     *
     * @param indexImgDTO
     * @return 轮播图列表
     */
    List<IndexImgVO> list(@Param("indexImg") IndexImgDTO indexImgDTO);

    /**
     * 根据轮播图id获取轮播图
     *
     * @param imgId 轮播图id
     * @return 轮播图
     */
    IndexImgVO getByImgId(@Param("imgId") Long imgId);

    /**
     * 保存轮播图
     *
     * @param indexImg 轮播图
     */
    void save(@Param("indexImg") IndexImg indexImg);

    /**
     * 更新轮播图
     *
     * @param indexImg 轮播图
     */
    void update(@Param("indexImg") IndexImg indexImg);

    /**
     * 根据轮播图id删除轮播图
     *
     * @param imgId  轮播图id
     * @param shopId 店铺id
     */
    void deleteByIdAndShopId(@Param("imgId") Long imgId, @Param("shopId") Long shopId);

    /**
     * 根据店铺id，获取轮播图列表
     *
     * @param shopId  店铺id
     * @param imgType 图片类型
     * @return 轮播图列表
     */
    List<IndexImgVO> getListByShopId(@Param("shopId") Long shopId, @Param("imgType") Integer imgType);

    /**
     * 解绑轮播图商品
     *
     * @param shopId
     * @param spuIds
     */
    void offlineSpuBySpuIds(@Param("shopId") Long shopId, @Param("spuIds") List<Long> spuIds);

    /**
     * 根据参数获取轮播图数量
     * @param indexImg 轮播图
     * @return 轮播图数量
     */
    int countByParam(@Param("indexImg") IndexImg indexImg);

    /**
     * 根据商品id，切换轮播图类型
     * @param spuId 商品id
     * @return 轮播图列表
     */
    void batchUpdate(@Param("spuId") Long spuId);
}
