/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.api.auth.feign;

import com.tmerclub.cloud.api.auth.bo.UidInfoBO;
import com.tmerclub.cloud.api.auth.constant.SysTypeEnum;
import com.tmerclub.cloud.api.auth.dto.AuthAccountDTO;
import com.tmerclub.cloud.api.auth.dto.AuthAccountUserDTO;
import com.tmerclub.cloud.api.auth.dto.AuthAccountWithSocialDTO;
import com.tmerclub.cloud.api.auth.dto.AuthSocialDTO;
import com.tmerclub.cloud.api.auth.vo.AuthAccountVO;
import com.tmerclub.cloud.api.auth.vo.TokenInfoVO;
import com.tmerclub.cloud.common.response.ServerResponseEntity;

import java.util.List;

/**
 * @author FrozenWatermelon
 * @date 2020/9/22
 */
public interface AccountFeignClient {

    /**
     * 保存统一账户
     * @param authAccountDTO 账户信息
     * @return Long uid
     */
    ServerResponseEntity<Long> save(AuthAccountDTO authAccountDTO);

    /**
     * 更新统一账户
     * @param authAccountDTO 账户信息
     * @return void
     */
    ServerResponseEntity<Void> update(AuthAccountDTO authAccountDTO);

    /**
     * 更新账户状态
     * @param authAccountDTO 账户信息
     * @return void
     */
    ServerResponseEntity<Void> updateAuthAccountStatus(AuthAccountDTO authAccountDTO);

    /**
     * 根据用户id和系统类型删除用户
     * @param userId 用户id
     * @return void
     */
    ServerResponseEntity<Void> deleteByUserIdAndSysType(Long userId);

    /**
     * 删除账号（通过租户id和系统类型）
     * @param tenantId 租户id
     * @param sysType 系统类型
     */
    void deleteByTenantIdAndSysType(Long tenantId, Integer sysType);

    /**
     * 根据手机号获取存在用户的数量
     * @param mobile 手机号
     * @return 数量
     */
    ServerResponseEntity<Integer> countByMobile(String mobile);


    /**
     * 根据用户id和系统类型获取用户信息
     * @param userId 用户id
     * @param sysType 系统类型
     * @return void
     */
    ServerResponseEntity<AuthAccountVO> getByUserIdAndSysType(Long userId, Integer sysType);

    /**
     * 根据用户id和系统类型获取用户信息
     * @param userId 用户id
     * @param sysType 系统类型
     * @return void
     */
    ServerResponseEntity<AuthAccountVO> getInsiderDataByUserIdAndSysType(Long userId, Integer sysType);

    /**
     * 根据用户id和系统类型获取用户信息
     * @param sysType 系统类型
     * @param isAdmin 是否管理员
     * @param tenantId 租户id
     * @return void
     */
    ServerResponseEntity<AuthAccountVO> getBySysTypeAndIsAdminAndTenantId(Integer sysType, Integer isAdmin, Long tenantId);

    /**
     * 保存统一账户
     * @param authAccountWithSocialDTO 账户信息 和社交账号信息
     * @return uid
     */
    ServerResponseEntity<Long> saveAccountWithSocial(AuthAccountWithSocialDTO authAccountWithSocialDTO);

    /**
     * 保存用户信息，生成token，返回前端
     * @param uidInfoBO 账户信息 和社交账号信息
     * @return uid
     */
    ServerResponseEntity<TokenInfoVO> storeTokenAndGetVo(UidInfoBO uidInfoBO);

    /**
     * 根据账号信息（账号名、手机号、邮箱），获取对应的用户列表
     * @param userName
     * @param phone
     * @param email
     * @param sysType
     * @return 用户账号信息列表
     */
    ServerResponseEntity<List<AuthAccountVO>> listByUserAccount(String userName, String phone
            , String email, SysTypeEnum sysType);

    /**
     * 根据用户id及系统id修改用户手机号
     * @param authAccountDTO 用户信息
     * @return void
     */
    ServerResponseEntity<Void> updateUserMobile(AuthAccountDTO authAccountDTO);

    /**
     * 根据用户id和系统类型，获取账号列表信息
     * @param userIds
     * @param systemType
     * @return
     */
    ServerResponseEntity<List<AuthAccountVO>> listUserByUserIdsAndType(List<Long> userIds, Integer systemType);

    /**
     * 根据用户ids，或者电话号码，系统类型获取用户列表
     * @param userIds
     * @param phone
     * @param sysType
     * @return
     */
    ServerResponseEntity<List<AuthAccountVO>> listByUserIdsAndPhoneAndType(List<Long> userIds,
                                                                           String phone,
                                                                           Integer sysType);

    /**
     * userId，系统类型获取用户信息
     * @param userId
     * @param sysType
     * @return
     */
    ServerResponseEntity<AuthAccountVO> getByUserIdAndType(Long userId,
                                                           Integer sysType);


    /**
     * 根据用户id与用户类型更新用户信息
     * @param uidInfoBO 新的用户信息
     * @param userId 用户id
     * @param sysType 用户类型
     * @return
     */
    ServerResponseEntity<Void> updateUserInfoByUserIdAndSysType(UidInfoBO uidInfoBO, Long userId, Integer sysType);

    /**
     * 根据手机号和用户类型获取存在用户的数量
     * @param mobile 手机号
     * @param sysType 用户类型
     * @return 数量
     */
    ServerResponseEntity<Integer> countByMobileAndSysType(String mobile, Integer sysType);

    /**
     * 修改用户状态，删除用户登录token信息,使用户下线
     * @param authAccountDTO 账户信息
     * @return void
     */
    ServerResponseEntity<Void> updateAuthAccountStatusAndDeleteUserToken(AuthAccountDTO authAccountDTO);

    /**
     * 根据用户id获取用户的微信小程序的userid集合
     * @param userId 手机号
     * @return 第三方微信小程序的userid集合
     */
    ServerResponseEntity<List<String>> getBizUserIdListByUserId(Long userId);

    /**
     * 根据临时tempUid获取用户的微信小程序的userid集合
     * @param tempUid
     * @return
     */
    ServerResponseEntity<AuthSocialDTO> getBizUserIdByTempUid(String tempUid);

    /**
     * 批量注册用户信息
     * @param authAccountUserDTO
     * @return
     */
    ServerResponseEntity<String> batchRegisterAccount(AuthAccountUserDTO authAccountUserDTO);

    /**
     * 根据租户id查询店铺信息
     * @param tenantId
     * @param sysType
     * @return
     */
    ServerResponseEntity<AuthAccountVO> getAccountInfoByTenantId(Long tenantId, Integer sysType);

    /**
     * 验证手机号/邮箱/用户名是否存在
     * @param authAccountDTO 账户信息
     * @return Boolean true验证通过,该账号不存在 false验证不通过，该账号存在
     */
    ServerResponseEntity<Boolean> verifyAccount(AuthAccountDTO authAccountDTO);

    /**
     * 根据用户名和用户类型获取存在用户的数量
     * @param userName 用户名
     * @param sysType 用户类型
     * @return 数量
     */
    ServerResponseEntity<Integer> countByUserNameAndSysType(String userName, Integer sysType);

    /**
     * 获取已存在的用户名称
     * @param names
     * @return
     */
    ServerResponseEntity<List<String>> listUserNameByNames(List<String> names);

    /**
     * 更新未开店成功账号的token用户信息
     * @param uidInfoBO
     * @return
     */
    ServerResponseEntity<Void> updateNoAuditUidInfo(UidInfoBO uidInfoBO);

}
