/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.api.user.feign;

import com.tmerclub.cloud.api.user.bo.UidAndUserIdBO;
import com.tmerclub.cloud.api.user.dto.CustomerReqDTO;
import com.tmerclub.cloud.api.user.dto.MemberReqDTO;
import com.tmerclub.cloud.api.user.dto.UserRegisterDTO;
import com.tmerclub.cloud.api.user.vo.MemberContributeRespVO;
import com.tmerclub.cloud.api.user.vo.MemberCountVO;
import com.tmerclub.cloud.api.user.vo.MemberTrendRespVO;
import com.tmerclub.cloud.api.user.vo.UserApiVO;
import com.tmerclub.cloud.common.response.ServerResponseEntity;
import jakarta.validation.Valid;

import java.util.Date;
import java.util.List;

/**
 * 用户信息feign连接
 * @author FrozenWatermelon
 * @date 2020/12/07
 */
public interface UserFeignClient {

    /**
     * 根据用户id列表，获取用户信息
     * @param userIds 用户id列表
     * @return 用户列表信息
     */
    ServerResponseEntity<List<UserApiVO>> getUserByUserIds(List<Long> userIds);

    /**
     * 根据搜索条件查询用户列表
     * @param userApiVO
     * @return 用户列表信息
     */
    ServerResponseEntity<List<UserApiVO>> getUserByParam(UserApiVO userApiVO);

    /**
     * 根据用户id获取用户详细信息
     * @param userId 用户id
     * @return 用户详细信息
     */
    ServerResponseEntity<UserApiVO> getUserAndOpenIdsByUserId(Long userId);

    /**
     * 获取用户数据
     * @param userId
     * @return
     */
    ServerResponseEntity<UserApiVO> getUserData(Long userId);

    /**
     * 跳过鉴权获取用户数据
     * @param userId 用户id
     * @return UserApiVO 用户信息
     */
    ServerResponseEntity<UserApiVO> getInsiderUserData(Long userId);


    /**
     * 获取用户分析数据
     * @param param 筛选参数
     * @return 用户分析数据
     */
    ServerResponseEntity<List<Long>> getMeetConditionsUserIds(MemberReqDTO param);

    /**
     * --------------数据分析相关sql--------------
     * 根据条件参数，统计会员数量
     * @param param 参数
     * @return 数量
     */
    ServerResponseEntity<MemberContributeRespVO> getMemberContributeByParam(MemberReqDTO param);

    /**
     * 筛选时间类的每一天注册的会员数数据
     * @param param 参数
     * @return 会员数数据
     */
    ServerResponseEntity<Integer> getMemberTrend(MemberReqDTO param);

    /**
     * 根据手机号码查询用户数量
     * @param mobile 手机号码
     * @return 用户数量
     */
    ServerResponseEntity<List<UserApiVO>> countUserByMobile(String mobile);

    /**
     * 取消过期优惠券的绑定
     * @param couponIds 优惠券id列表
     * @return
     */
    ServerResponseEntity<Void> cancelBindingCoupons(List<Long> couponIds);

    /**
     * 用户注册并绑定第三方账号
     * @param userRegisterDTO 注册信息
     * @return
     */
    ServerResponseEntity<UidAndUserIdBO> save(@Valid UserRegisterDTO userRegisterDTO);

    /**
     * 统计时间段内的新增用户数量
     * @param customerReqDTO
     * @return
     */
    ServerResponseEntity<Integer> countNewUserByTime(CustomerReqDTO customerReqDTO);


    /**
     * 获取时间段内的新增用户id列表
     * @param customerReqDTO
     * @return
     */
    List<Long> listNewUserIdByTime(CustomerReqDTO customerReqDTO);

    /**
     * 获取指定用户列表
     * @param userIds
     * @return
     */
    List<UserApiVO> listUserByUserIds(List<Long> userIds);

    /**
     * 创建通联支付会员(个人会员)
     * @return
     */
    ServerResponseEntity<Void> createAllinpayMember();

    /**
     * 统计会员数量（普通会员、付费会员）
     * @param customerReqDTO
     * @return
     */
    ServerResponseEntity<MemberTrendRespVO> countUserNumByType(CustomerReqDTO customerReqDTO);

    /**
     * 统计累计用户数、新增用户数、付费会员数
     * @param memberReqDTO
     * @return
     */
    MemberCountVO statisticsUserNum(MemberReqDTO memberReqDTO);

    /**
     * 获取第一个用户的注册时间
     * @return
     */
    ServerResponseEntity<Date> getInitialRegistrationTime();

}
