/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.common.order.vo;

import com.tmerclub.cloud.common.vo.BaseVO;
import io.swagger.v3.oas.annotations.media.Schema;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 订单信息
 *
 * @author FrozenWatermelon
 * @date 2020-12-05 14:13:50
 */
public class OrderVO extends BaseVO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    @Schema(description = "订单ID")
    private Long orderId;

    @Schema(description = "店铺id")
    private Long shopId;

    @Schema(description = "用户ID")
    private Long userId;

    @Schema(description = "用户订单地址Id")
    private Long orderAddrId;

    @Schema(description = "店铺名称")
    private String shopName;

    @Schema(description = "买家昵称")
    private String nickName;

    @Schema(description = "总值")
    private Long total;

    @Schema(description = "实际总值")
    private Long actualTotal;

    @Schema(description = "订单使用积分")
    private Long orderScore;

    @Schema(description = "买家备注")
    private String remarks;

    @Schema(description = "卖家备注")
    private String shopRemarks;

    @Schema(description = "订单状态 1:待付款 2:待发货(待自提) 3:待收货(已发货) 5:成功 6:失败 7:待成团")
    private Integer status;

    @Schema(description = "支付方式 请参考枚举PayType")
    private Integer payType;

    @Schema(description = "配送类型 1:快递 2:自提 3：无需快递 4同城配送")
    private Integer deliveryType;

    @Schema(description = "订单类型1团购订单 2秒杀订单 3积分订单")
    private Integer orderType;

    @Schema(description = "是否为虚拟商品订单 1.是 0.否")
    private Integer orderMold;

    @Schema(description = "订单关闭原因 1-超时未支付 2-退款关闭 4-买家取消 15-已通过货到付款交易")
    private Integer closeType;

    @Schema(description = "订单商品总数")
    private Integer allCount;

    @Schema(description = "付款时间")
    private Date payTime;

    @Schema(description = "发货时间")
    private Date deliveryTime;

    @Schema(description = "完成时间")
    private Date finallyTime;

    @Schema(description = "取消时间")
    private Date cancelTime;

    @Schema(description = "预售发货时间")
    private Date bookTime;

    @Schema(description = "是否已支付，1.已支付0.未支付")
    private Integer isPayed;

    @Schema(description = "用户订单删除状态，0：没有删除， 1：回收站， 2：永久删除")
    private Integer deleteStatus;

    @Schema(description = "订单退款状态（1:申请退款 2:退款成功 3:部分退款成功 4:退款失败）")
    private Integer refundStatus;

    @Schema(description = "订单运费")
    private Long freightAmount;

    @Schema(description = "店铺订单运费")
    private Long shopChangeFreeAmount;

    @Schema(description = "积分抵扣金额")
    private Long scoreAmount;

    @Schema(description = "会员折扣金额")
    private Long memberAmount;

    @Schema(description = "平台优惠券优惠金额")
    private Long platformCouponAmount;

    @Schema(description = "商家优惠券优惠金额")
    private Long shopCouponAmount;

    @Schema(description = "满减优惠金额")
    private Long discountAmount;

    @Schema(description = "分销金额")
    private Long distributionAmount;

    @Schema(description = "平台优惠金额")
    private Long platformAmount;

    @Schema(description = "平台运费减免金额")
    private Long platformFreeFreightAmount;

    @Schema(description = "优惠总额")
    private Long reduceAmount;

    @Schema(description = "订单项")
    private List<OrderItemVO> orderItems;

    @Schema(description = "订单地址")
    private OrderAddrVO orderAddr;

    @Schema(description = "物流Id")
    private Long dvyId;

    @Schema(description = "物流单号")
    private String dvyFlowId;

    @Schema(description = "商家运费减免金额")
    private Long freeFreightAmount;

    @Schema(description = "消费者联系方式")
    private String mobile;

    @Schema(description = "供应商联系方式")
    private String supplierMobile;

    @Schema(description = "供应商id")
    private Long supplierId;

    @Schema(description = "供应商商品发货方式 1.供应商发货 2.仓库发货")
    private Integer supplierDeliveryType;

    @Schema(description = "否待采购 1.是 0.否")
    private Integer waitPurchase;

    @Schema(description = "是否已经进行结算")
    private Integer isSettled;

    @Schema(description = "平台佣金")
    private Long platformCommission;

    @Schema(description = "商家采购价差（需要商家从结算金额补上采购价的金额）")
    private Long purchaseSpreadAmount;
    @Schema(description = "预售类型  -1：未开启 0.全款预售 1.定金预售")
    private Integer preSaleType;

    @Schema(description = "预售信息")
    private OrderPreSaleInfoVO orderPreSaleInfoVO;

    @Schema(description = "供应商名称")
    private String supplierName;

    @Schema(description = "用户自提信息")
    private OrderSelfStationVO orderSelfStationVO;

    @Schema(description = "自提点id")
    private Long stationId;

    @Schema(description = "套餐优惠金额")
    private Long shopComboAmount;

    @Schema(description = "核销次数 -1.多次核销 0.无需核销 1.单次核销")
    private Integer writeOffNum;

    @Schema(description = "多次核销次数 -1.无限次")
    private Integer writeOffMultipleCount;

    @Schema(description = "核销开始时间")
    private Date writeOffStart;

    @Schema(description = "核销结束时间")
    private Date writeOffEnd;

    @Schema(description = "订单核销状态 0.待核销 1.核销完成")
    private Integer writeOffStatus;

    @Schema(description = "核销订单")
    private List<OrderVirtualInfoVO> orderVirtualInfoList;
    /**
     * 支付系统类型
     */
    private Integer paySysType;

    /**
     * 订单被核销次数
     */
    private Integer writeOffCount;

    /**
     * 主单号,在组合商品拆单时会存在
     */
    private Long mainOrderId;

    public Long getMainOrderId() {
        return mainOrderId;
    }

    public void setMainOrderId(Long mainOrderId) {
        this.mainOrderId = mainOrderId;
    }
    public Integer getWriteOffCount() {
        return writeOffCount;
    }

    public void setWriteOffCount(Integer writeOffCount) {
        this.writeOffCount = writeOffCount;
    }

    public Integer getPaySysType() {
        return paySysType;
    }

    public void setPaySysType(Integer paySysType) {
        this.paySysType = paySysType;
    }

    public List<OrderVirtualInfoVO> getOrderVirtualInfoList() {
        return orderVirtualInfoList;
    }

    public void setOrderVirtualInfoList(List<OrderVirtualInfoVO> orderVirtualInfoList) {
        this.orderVirtualInfoList = orderVirtualInfoList;
    }

    public Integer getWriteOffNum() {
        return writeOffNum;
    }

    public void setWriteOffNum(Integer writeOffNum) {
        this.writeOffNum = writeOffNum;
    }

    public Integer getWriteOffMultipleCount() {
        return writeOffMultipleCount;
    }

    public void setWriteOffMultipleCount(Integer writeOffMultipleCount) {
        this.writeOffMultipleCount = writeOffMultipleCount;
    }

    public Date getWriteOffStart() {
        return writeOffStart;
    }

    public void setWriteOffStart(Date writeOffStart) {
        this.writeOffStart = writeOffStart;
    }

    public Date getWriteOffEnd() {
        return writeOffEnd;
    }

    public void setWriteOffEnd(Date writeOffEnd) {
        this.writeOffEnd = writeOffEnd;
    }

    public Integer getWriteOffStatus() {
        return writeOffStatus;
    }

    public void setWriteOffStatus(Integer writeOffStatus) {
        this.writeOffStatus = writeOffStatus;
    }

    public Long getShopComboAmount() {
        return shopComboAmount;
    }

    public void setShopComboAmount(Long shopComboAmount) {
        this.shopComboAmount = shopComboAmount;
    }

    public Long getStationId() {
        return stationId;
    }

    public void setStationId(Long stationId) {
        this.stationId = stationId;
    }

    public OrderSelfStationVO getOrderSelfStationVO() {
        return orderSelfStationVO;
    }

    public void setOrderSelfStationVO(OrderSelfStationVO orderSelfStationVO) {
        this.orderSelfStationVO = orderSelfStationVO;
    }

    public Long getOrderId() {
        return orderId;
    }

    public void setOrderId(Long orderId) {
        this.orderId = orderId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public Long getOrderAddrId() {
        return orderAddrId;
    }

    public void setOrderAddrId(Long orderAddrId) {
        this.orderAddrId = orderAddrId;
    }

    public String getShopName() {
        return shopName;
    }

    public void setShopName(String shopName) {
        this.shopName = shopName;
    }

    public String getNickName() {
        return nickName;
    }

    public void setNickName(String nickName) {
        this.nickName = nickName;
    }

    public Long getTotal() {
        return total;
    }

    public void setTotal(Long total) {
        this.total = total;
    }

    public Long getActualTotal() {
        return actualTotal;
    }

    public void setActualTotal(Long actualTotal) {
        this.actualTotal = actualTotal;
    }

    public Long getOrderScore() {
        return orderScore;
    }

    public void setOrderScore(Long orderScore) {
        this.orderScore = orderScore;
    }

    public String getRemarks() {
        return remarks;
    }

    public void setRemarks(String remarks) {
        this.remarks = remarks;
    }

    public String getShopRemarks() {
        return shopRemarks;
    }

    public void setShopRemarks(String shopRemarks) {
        this.shopRemarks = shopRemarks;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getPayType() {
        return payType;
    }

    public void setPayType(Integer payType) {
        this.payType = payType;
    }

    public Integer getDeliveryType() {
        return deliveryType;
    }

    public void setDeliveryType(Integer deliveryType) {
        this.deliveryType = deliveryType;
    }

    public Integer getOrderType() {
        return orderType;
    }

    public void setOrderType(Integer orderType) {
        this.orderType = orderType;
    }

    public Integer getOrderMold() {
        return orderMold;
    }

    public void setOrderMold(Integer orderMold) {
        this.orderMold = orderMold;
    }

    public Integer getCloseType() {
        return closeType;
    }

    public void setCloseType(Integer closeType) {
        this.closeType = closeType;
    }

    public Integer getAllCount() {
        return allCount;
    }

    public void setAllCount(Integer allCount) {
        this.allCount = allCount;
    }

    public Date getPayTime() {
        return payTime;
    }

    public void setPayTime(Date payTime) {
        this.payTime = payTime;
    }

    public Date getDeliveryTime() {
        return deliveryTime;
    }

    public void setDeliveryTime(Date deliveryTime) {
        this.deliveryTime = deliveryTime;
    }

    public Date getFinallyTime() {
        return finallyTime;
    }

    public void setFinallyTime(Date finallyTime) {
        this.finallyTime = finallyTime;
    }

    public Date getCancelTime() {
        return cancelTime;
    }

    public void setCancelTime(Date cancelTime) {
        this.cancelTime = cancelTime;
    }

    public Date getBookTime() {
        return bookTime;
    }

    public void setBookTime(Date bookTime) {
        this.bookTime = bookTime;
    }

    public Integer getIsPayed() {
        return isPayed;
    }

    public void setIsPayed(Integer isPayed) {
        this.isPayed = isPayed;
    }

    public Integer getDeleteStatus() {
        return deleteStatus;
    }

    public void setDeleteStatus(Integer deleteStatus) {
        this.deleteStatus = deleteStatus;
    }

    public Integer getRefundStatus() {
        return refundStatus;
    }

    public void setRefundStatus(Integer refundStatus) {
        this.refundStatus = refundStatus;
    }

    public Long getFreightAmount() {
        return freightAmount;
    }

    public void setFreightAmount(Long freightAmount) {
        this.freightAmount = freightAmount;
    }

    public Long getShopChangeFreeAmount() {
        return shopChangeFreeAmount;
    }

    public void setShopChangeFreeAmount(Long shopChangeFreeAmount) {
        this.shopChangeFreeAmount = shopChangeFreeAmount;
    }

    public Long getScoreAmount() {
        return scoreAmount;
    }

    public void setScoreAmount(Long scoreAmount) {
        this.scoreAmount = scoreAmount;
    }

    public Long getMemberAmount() {
        return memberAmount;
    }

    public void setMemberAmount(Long memberAmount) {
        this.memberAmount = memberAmount;
    }

    public Long getPlatformCouponAmount() {
        return platformCouponAmount;
    }

    public void setPlatformCouponAmount(Long platformCouponAmount) {
        this.platformCouponAmount = platformCouponAmount;
    }

    public Long getShopCouponAmount() {
        return shopCouponAmount;
    }

    public void setShopCouponAmount(Long shopCouponAmount) {
        this.shopCouponAmount = shopCouponAmount;
    }

    public Long getDiscountAmount() {
        return discountAmount;
    }

    public void setDiscountAmount(Long discountAmount) {
        this.discountAmount = discountAmount;
    }

    public Long getDistributionAmount() {
        return distributionAmount;
    }

    public void setDistributionAmount(Long distributionAmount) {
        this.distributionAmount = distributionAmount;
    }

    public Long getPlatformAmount() {
        return platformAmount;
    }

    public void setPlatformAmount(Long platformAmount) {
        this.platformAmount = platformAmount;
    }

    public Long getPlatformFreeFreightAmount() {
        return platformFreeFreightAmount;
    }

    public void setPlatformFreeFreightAmount(Long platformFreeFreightAmount) {
        this.platformFreeFreightAmount = platformFreeFreightAmount;
    }

    public Long getReduceAmount() {
        return reduceAmount;
    }

    public void setReduceAmount(Long reduceAmount) {
        this.reduceAmount = reduceAmount;
    }

    public List<OrderItemVO> getOrderItems() {
        return orderItems;
    }

    public void setOrderItems(List<OrderItemVO> orderItems) {
        this.orderItems = orderItems;
    }

    public OrderAddrVO getOrderAddr() {
        return orderAddr;
    }

    public void setOrderAddr(OrderAddrVO orderAddr) {
        this.orderAddr = orderAddr;
    }

    public Long getDvyId() {
        return dvyId;
    }

    public void setDvyId(Long dvyId) {
        this.dvyId = dvyId;
    }

    public String getDvyFlowId() {
        return dvyFlowId;
    }

    public void setDvyFlowId(String dvyFlowId) {
        this.dvyFlowId = dvyFlowId;
    }

    public Long getFreeFreightAmount() {
        return freeFreightAmount;
    }

    public void setFreeFreightAmount(Long freeFreightAmount) {
        this.freeFreightAmount = freeFreightAmount;
    }

    public String getMobile() {
        return mobile;
    }

    public void setMobile(String mobile) {
        this.mobile = mobile;
    }

    public String getSupplierMobile() {
        return supplierMobile;
    }

    public void setSupplierMobile(String supplierMobile) {
        this.supplierMobile = supplierMobile;
    }

    public Long getSupplierId() {
        return supplierId;
    }

    public void setSupplierId(Long supplierId) {
        this.supplierId = supplierId;
    }

    public Integer getSupplierDeliveryType() {
        return supplierDeliveryType;
    }

    public void setSupplierDeliveryType(Integer supplierDeliveryType) {
        this.supplierDeliveryType = supplierDeliveryType;
    }

    public Integer getWaitPurchase() {
        return waitPurchase;
    }

    public void setWaitPurchase(Integer waitPurchase) {
        this.waitPurchase = waitPurchase;
    }

    public Integer getIsSettled() {
        return isSettled;
    }

    public void setIsSettled(Integer isSettled) {
        this.isSettled = isSettled;
    }

    public Long getPlatformCommission() {
        return platformCommission;
    }

    public void setPlatformCommission(Long platformCommission) {
        this.platformCommission = platformCommission;
    }

    public Long getPurchaseSpreadAmount() {
        return purchaseSpreadAmount;
    }

    public void setPurchaseSpreadAmount(Long purchaseSpreadAmount) {
        this.purchaseSpreadAmount = purchaseSpreadAmount;
    }

    public Integer getPreSaleType() {
        return preSaleType;
    }

    public void setPreSaleType(Integer preSaleType) {
        this.preSaleType = preSaleType;
    }

    public OrderPreSaleInfoVO getOrderPreSaleInfoVO() {
        return orderPreSaleInfoVO;
    }

    public void setOrderPreSaleInfoVO(OrderPreSaleInfoVO orderPreSaleInfoVO) {
        this.orderPreSaleInfoVO = orderPreSaleInfoVO;
    }

    public String getSupplierName() {
        return supplierName;
    }

    public void setSupplierName(String supplierName) {
        this.supplierName = supplierName;
    }

    @Override
    public String toString() {
        return "OrderVO{" +
                "orderId=" + orderId +
                ", shopId=" + shopId +
                ", userId=" + userId +
                ", orderAddrId=" + orderAddrId +
                ", shopName='" + shopName + '\'' +
                ", nickName='" + nickName + '\'' +
                ", total=" + total +
                ", actualTotal=" + actualTotal +
                ", orderScore=" + orderScore +
                ", remarks='" + remarks + '\'' +
                ", shopRemarks='" + shopRemarks + '\'' +
                ", status=" + status +
                ", payType=" + payType +
                ", deliveryType=" + deliveryType +
                ", orderType=" + orderType +
                ", orderMold=" + orderMold +
                ", closeType=" + closeType +
                ", allCount=" + allCount +
                ", payTime=" + payTime +
                ", deliveryTime=" + deliveryTime +
                ", finallyTime=" + finallyTime +
                ", cancelTime=" + cancelTime +
                ", bookTime=" + bookTime +
                ", isPayed=" + isPayed +
                ", deleteStatus=" + deleteStatus +
                ", refundStatus=" + refundStatus +
                ", freightAmount=" + freightAmount +
                ", shopChangeFreeAmount=" + shopChangeFreeAmount +
                ", scoreAmount=" + scoreAmount +
                ", memberAmount=" + memberAmount +
                ", platformCouponAmount=" + platformCouponAmount +
                ", shopCouponAmount=" + shopCouponAmount +
                ", discountAmount=" + discountAmount +
                ", distributionAmount=" + distributionAmount +
                ", platformAmount=" + platformAmount +
                ", platformFreeFreightAmount=" + platformFreeFreightAmount +
                ", reduceAmount=" + reduceAmount +
                ", orderItems=" + orderItems +
                ", orderAddr=" + orderAddr +
                ", dvyId=" + dvyId +
                ", dvyFlowId='" + dvyFlowId + '\'' +
                ", freeFreightAmount=" + freeFreightAmount +
                ", mobile='" + mobile + '\'' +
                ", supplierMobile='" + supplierMobile + '\'' +
                ", supplierId=" + supplierId +
                ", supplierDeliveryType=" + supplierDeliveryType +
                ", waitPurchase=" + waitPurchase +
                ", isSettled=" + isSettled +
                ", platformCommission=" + platformCommission +
                ", purchaseSpreadAmount=" + purchaseSpreadAmount +
                ", preSaleType=" + preSaleType +
                ", orderPreSaleInfoVO=" + orderPreSaleInfoVO +
                ", supplierName='" + supplierName + '\'' +
                ", orderSelfStationVO=" + orderSelfStationVO +
                ", stationId=" + stationId +
                ", shopComboAmount=" + shopComboAmount +
                ", writeOffNum=" + writeOffNum +
                ", writeOffMultipleCount=" + writeOffMultipleCount +
                ", writeOffStart=" + writeOffStart +
                ", writeOffEnd=" + writeOffEnd +
                ", writeOffStatus=" + writeOffStatus +
                ", orderVirtualInfoList=" + orderVirtualInfoList +
                ", paySysType=" + paySysType +
                ", writeOffCount=" + writeOffCount +
                ", mainOrderId=" + mainOrderId +
                '}';
    }
}
