/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.group.bo.mongo;

import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.CompoundIndex;
import org.springframework.data.mongodb.core.index.CompoundIndexes;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;

/**
 * Mongodb拼团订单表
 * @author TRACK
 */
@CompoundIndexes({
        @CompoundIndex(name = "user_idx", def = "{'userId':1}"),
        @CompoundIndex(name = "shop_idx", def = "{'shopId':1}"),
        @CompoundIndex(name = "group_activity_idx", def = "{'groupActivityId':1}")
})
@Document("group_order")
public class MongoGroupOrderBO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;

    @Id
    @Schema(description = "拼团订单id", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long groupOrderId;

    @Schema(description = "拼团活动id")
    private Long groupActivityId;

    @Schema(description = "店铺id")
    private Long shopId;

    @Schema(description = "拼团团队id")
    private Long groupTeamId;

    @Schema(description = "user_id(当user_id为0时标识为机器人)")
    private Long userId;

    @Schema(description = "创建时间", requiredMode = Schema.RequiredMode.REQUIRED)
    private Date createTime;

    @Schema(description = "更新时间")
    private Date updateTime;

    @Schema(description = "身份标识(0:成员  1:团长)")
    private Integer identityType;

    @Schema(description = "活动商品金额")
    private Long activityProdPrice;

    @Schema(description = "支付金额")
    private Long payPrice;

    @Schema(description = "订单编号")
    private Long orderId;

    @Schema(description = "状态(0:待支付、1:支付成功、-1:失效)")
    private Integer status;

    @Schema(description = "商品数量")
    private Integer count;

    public Long getGroupOrderId() {
        return groupOrderId;
    }

    public void setGroupOrderId(Long groupOrderId) {
        this.groupOrderId = groupOrderId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Long getGroupTeamId() {
        return groupTeamId;
    }

    public void setGroupTeamId(Long groupTeamId) {
        this.groupTeamId = groupTeamId;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public Integer getIdentityType() {
        return identityType;
    }

    public void setIdentityType(Integer identityType) {
        this.identityType = identityType;
    }

    public Long getActivityProdPrice() {
        return activityProdPrice;
    }

    public void setActivityProdPrice(Long activityProdPrice) {
        this.activityProdPrice = activityProdPrice;
    }

    public Long getPayPrice() {
        return payPrice;
    }

    public void setPayPrice(Long payPrice) {
        this.payPrice = payPrice;
    }

    public Long getOrderId() {
        return orderId;
    }

    public void setOrderId(Long orderId) {
        this.orderId = orderId;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Long getGroupActivityId() {
        return groupActivityId;
    }

    public void setGroupActivityId(Long groupActivityId) {
        this.groupActivityId = groupActivityId;
    }

    public Integer getCount() {
        return count;
    }

    public void setCount(Integer count) {
        this.count = count;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    @Override
    public String toString() {
        return "MongoGroupOrderBO{" +
                "groupOrderId=" + groupOrderId +
                ", groupActivityId=" + groupActivityId +
                ", shopId=" + shopId +
                ", groupTeamId=" + groupTeamId +
                ", userId=" + userId +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", identityType=" + identityType +
                ", activityProdPrice=" + activityProdPrice +
                ", payPrice=" + payPrice +
                ", orderId=" + orderId +
                ", status=" + status +
                ", count=" + count +
                '}';
    }
}
