/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.group.bo.mongo;

import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.CompoundIndex;
import org.springframework.data.mongodb.core.index.CompoundIndexes;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;

/**
 * Mongodb拼团团队表
 * @author TRACK
 */
@CompoundIndexes({
        @CompoundIndex(name = "shop_idx", def = "{'shopId':1}"),
        @CompoundIndex(name = "share_user_idx", def = "{'shareUserId':1}"),
        @CompoundIndex(name = "group_spu_idx", def = "{'groupSpuId':1}"),
        @CompoundIndex(name = "group_activity_idx", def = "{'groupActivityId':1}")
})
@Document("group_team")
public class MongoGroupTeamBO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;

    @Schema(description = "拼团团队id", requiredMode = Schema.RequiredMode.REQUIRED)
    @Id
    private Long groupTeamId;

    @Schema(description = "店铺id")
    private Long shopId;

    @Schema(description = "拼团活动id")
    private Long groupActivityId;

    @Schema(description = "创建时间", requiredMode = Schema.RequiredMode.REQUIRED)
    private Date createTime;

    @Schema(description = "更新时间")
    private Date updateTime;

    @Schema(description = "活动商品Id")
    private Long groupSpuId;

    @Schema(description = "已参团人数")
    private Integer joinNum;

    @Schema(description = "拼团状态(0:待成团，1:拼团中，2:拼团成功，3:拼团失败)")
    private Integer status;

    @Schema(description = "团队订单总额")
    private Long totalPrice;

    @Schema(description = "开始时间（团长支付成功时间）")
    private Date startTime;

    @Schema(description = "剩余时间")
    private Date endTime;

    @Schema(description = "团长user_Id")
    private Long shareUserId;

    public Long getGroupTeamId() {
        return groupTeamId;
    }

    public void setGroupTeamId(Long groupTeamId) {
        this.groupTeamId = groupTeamId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Long getGroupActivityId() {
        return groupActivityId;
    }

    public void setGroupActivityId(Long groupActivityId) {
        this.groupActivityId = groupActivityId;
    }

    public Long getGroupSpuId() {
        return groupSpuId;
    }

    public void setGroupSpuId(Long groupSpuId) {
        this.groupSpuId = groupSpuId;
    }

    public Integer getJoinNum() {
        return joinNum;
    }

    public void setJoinNum(Integer joinNum) {
        this.joinNum = joinNum;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Long getTotalPrice() {
        return totalPrice;
    }

    public void setTotalPrice(Long totalPrice) {
        this.totalPrice = totalPrice;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Long getShareUserId() {
        return shareUserId;
    }

    public void setShareUserId(Long shareUserId) {
        this.shareUserId = shareUserId;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    @Override
    public String toString() {
        return "MongoGroupTeamBO{" +
                ", groupTeamId=" + groupTeamId +
                ", shopId=" + shopId +
                ", groupActivityId=" + groupActivityId +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", groupSpuId=" + groupSpuId +
                ", joinNum=" + joinNum +
                ", status=" + status +
                ", totalPrice=" + totalPrice +
                ", startTime=" + startTime +
                ", endTime=" + endTime +
                ", shareUserId=" + shareUserId +
                '}';
    }
}
