/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.live.mapper;

import com.tmerclub.cloud.common.database.util.PageAdapter;
import com.tmerclub.cloud.live.dto.LiveRoomDTO;
import com.tmerclub.cloud.live.model.LiveRoom;
import com.tmerclub.cloud.live.vo.LiveRoomVO;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 直播间
 * @author TRACK
 */
public interface LiveRoomMapper {

    /**
     * 获取直播间列表
     * @param liveRoom
     * @return 直播间列表
     */
    List<LiveRoomVO> list(@Param("liveRoom") LiveRoomDTO liveRoom);

    /**
     * 根据直播间id获取直播间
     * @param roomId 直播间id
     * @return 直播间
     */
    LiveRoomVO getByRoomId(@Param("roomId") Long roomId);

    /**
     * 保存直播间
     * @param liveRoom 直播间
     */
    void save(@Param("liveRoom") LiveRoom liveRoom);

    /**
     * 更新直播间
     * @param liveRoom 直播间
     */
    void update(@Param("liveRoom") LiveRoom liveRoom);

    /**
     * 根据直播间id删除直播间
     * @param roomId
     */
    void deleteById(@Param("roomId") Long roomId);

    /**
     * 根据参数获取直播间数量
     * @param shopId
     * @param userId
     * @param roomId
     * @param startTime
     * @param endTime
     * @return
     */
    int countRoomByParam(@Param("shopId") Long shopId, @Param("userId") Long userId, @Param("roomId") Long roomId,
                         @Param("startTime") Date startTime, @Param("endTime") Date endTime);

    /**
     * 获取用户当前正在开播的直播间信息
     * @param userId
     * @param roomId
     * @return
     */
    LiveRoomVO getLiveRoomInfo(@Param("userId") Long userId, @Param("roomId") Long roomId);

    /**
     * 获取直播间列表
     * @param page
     * @param liveRoom
     * @return 直播间列表
     */
    List<LiveRoomVO> listRoomAndDetail(@Param("page") PageAdapter page, @Param("liveRoom") LiveRoomDTO liveRoom);

    /**
     * 获取直播间列表数
     * @param liveRoom
     * @return
     */
    long countRoomAndDetail(@Param("liveRoom") LiveRoomDTO liveRoom);

    /**
     * 更新直播间状态
     * @param liveStatus
     * @param roomIds
     */
    void updateLiveRoomStatus(@Param("liveStatus") Integer liveStatus, @Param("roomIds") List<Long> roomIds);

    /**
     * 根据时间获取对应状态的直播间列表
     * @param liveStatus
     * @return
     */
    List<Long> listLiveRoomByStatus(@Param("liveStatus") Integer liveStatus);

    /**
     * 获取需要暂停的直播间id
     * @param userId
     * @return
     */
    List<Long> listNeedStopLiveRoom(@Param("userId") Long userId);

    /**
     * 根据店铺id获取需要被结束的直播间
     * @param shopIds
     * @return
     */
    List<Long> listRoomIdsByShopId(@Param("shopIds") List<Long> shopIds);
}
