/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.product.listener;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.tmerclub.cloud.common.constant.Constant;
import com.tmerclub.cloud.product.constant.SpuExportError;
import com.tmerclub.cloud.product.service.SpuExcelService;
import com.tmerclub.cloud.product.vo.SpuExcelVO;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 商品excel导入监听
 * 有个很重要的点 SpuExcelListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
 *
 * @author YXF
 * @date 2021/3/11
 */

public class SpuExcelListener extends AnalysisEventListener<SpuExcelVO> {

    private SpuExcelService spuExcelService;
    private Map<Integer, List<String>> errorMap;
    private List<SpuExcelVO> list;
    private SpuExcelVO spuExcel;

    /**
     * 每隔1000条数据存储数据库，然后清理map ，方便内存回收
     */
    private static final int BATCH_COUNT = 3000;
    private static String seq;

    public SpuExcelListener() {
    }

    public SpuExcelListener(SpuExcelService spuExcelService, Map<Integer, List<String>> errorMap) {
        this.spuExcelService = spuExcelService;
        this.errorMap = errorMap;
        this.list = new ArrayList<>(Constant.INITIAL_CAPACITY);
    }

    /**
     * 这个每一条数据解析都会来调用
     */

    @Override
    public void invoke(SpuExcelVO spuExcelVO, AnalysisContext analysisContext) {

        boolean isSave = list.size() > BATCH_COUNT;
        if (isSave) {
            saveData();
        }

        list.add(spuExcelVO);
    }

    /**
     * 所有数据解析完成了 都会来调用
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        //确保所有数据都能入库
        saveData();
    }

    /**
     * 加上存储数据库
     */
    private void saveData() {
        try {
            spuExcelService.exportExcel(list, errorMap);
        } catch (Exception e) {
            List<String> list = errorMap.get(SpuExportError.OTHER.value());
            if (CollectionUtil.isEmpty(list)) {
                list = new ArrayList<>(0);
                errorMap.put(SpuExportError.OTHER.value(), list);
            }
            list.add(e.getMessage());
        }
        list.clear();
    }

}
