/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.user.mapper;

import com.tmerclub.cloud.common.order.vo.UserAddrVO;
import com.tmerclub.cloud.user.model.UserAddr;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 用户地址
 *
 * @author FrozenWatermelon
 * @date 2020-12-07 15:50:02
 */
public interface UserAddrMapper {

    /**
     * 获取用户地址列表
     *
     * @param userId
     * @return 用户地址列表
     */
    List<UserAddrVO> list(@Param("userId") Long userId);

    /**
     * 根据用户地址id获取用户地址
     *
     * @param userId 用户id
     * @param addrId 用户地址id
     * @return 用户地址
     */
    UserAddrVO getByAddrId(@Param("addrId") Long addrId, @Param("userId") Long userId);

    /**
     * 保存用户地址
     *
     * @param userAddr 用户地址
     */
    void save(@Param("userAddr") UserAddr userAddr);

    /**
     * 更新用户地址
     *
     * @param userAddr 用户地址
     */
    void update(@Param("userAddr") UserAddr userAddr);

    /**
     * 删除地址
     *
     * @param addrId 地址id
     * @param userId 用户id
     */
    void deleteById(@Param("addrId") Long addrId, @Param("userId") Long userId);


    /**
     * 移除用户默认地址
     *
     * @param userId
     */
    void removeDefaultUserAddr(@Param("userId") Long userId);

    /**
     * 用户地址的数量
     *
     * @param userId 用户id
     * @return 数量
     */
    int countByUserId(Long userId);

    /**
     * 通过用户id获取默认地址
     * @param userId 用户id
     * @return 默认地址
     */
    UserAddrVO getUserDefaultAddrByUserId(@Param("userId") Long userId);
}
