/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */
package com.tmerclub.cloud.user.service;

import com.tmerclub.cloud.common.database.dto.PageDTO;
import com.tmerclub.cloud.common.database.vo.PageVO;
import com.tmerclub.cloud.user.dto.UserRightsDTO;
import com.tmerclub.cloud.user.vo.UserRightsVO;

import java.util.List;

/**
 * 用户权益信息
 *
 * @author YXF
 * @date 2020-12-08 10:16:53
 */
public interface UserRightsService {

    /**
     * 分页获取用户权益信息列表
     * @param pageDTO 分页参数
     * @param userRightsDTO
     * @return 用户权益信息列表分页数据
     */
    PageVO<UserRightsVO> page(PageDTO pageDTO, UserRightsDTO userRightsDTO);


    /**
     * 根据用户权益信息id获取用户权益信息
     *
     * @param rightsId 用户权益信息id
     * @return 用户权益信息
     */
    UserRightsVO getByRightsId(Long rightsId);

    /**
     * 保存用户权益信息
     * @param userRights 用户权益信息
     */
    void save(UserRightsDTO userRights);

    /**
     * 更新用户权益信息
     * @param userRights 用户权益信息
     */
    void update(UserRightsDTO userRights);

    /**
     * 根据用户权益信息id删除用户权益信息
     * @param rightsId
     */
    void deleteById(Long rightsId);

    /**
     * 获取权益列表
     * @param userRights
     * @return
     */
    List<UserRightsVO> list(UserRightsDTO userRights);

    /**
     * 删除权益缓存
     * @param rightsId 权益id
     */
    void removeRightsCache(Long rightsId);

    /**
     * 删除去重后的权益列表缓存
     * @param levelType
     */
    void removeRightsByLevelTypeCache(Integer levelType);

    /**
     * 根据会员类型获取去重后的权益列表
     * @param levelType
     * @param level
     * @return
     */
    List<UserRightsVO> listRightsByLevelType(Integer levelType, Integer level);

    /**
     * 取消过期优惠券的绑定
     * @param couponIds
     */
    void cancelBindingCoupons(List<Long> couponIds);

    /**
     * 根据等级信息获取对应的等级权益
     * @param level
     * @param levelType
     * @return
     */
    List<UserRightsVO> listRightByLevel(Integer level, Integer levelType);
}
